`define NUM_CORE 10

module oc_dma_axi64_top(handsome_clk, handsome_din_top, handsome_dout_top, clk, pclken, reset);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input clk;
	input pclken;
	input reset;

localparam input_width = 187;
localparam output_width = 255;
localparam reducer_log4 = 4;
localparam reducer_log4_top = 2;

wire [`NUM_CORE-1:0] scan_en;
wire [`NUM_CORE-1:0] psel;
wire [`NUM_CORE-1:0] penable;
wire [`NUM_CORE-1:0] pwrite;
wire [`NUM_CORE-1:0] AWREADY0;
wire [`NUM_CORE-1:0] WREADY0;
wire [`NUM_CORE-1:0] BID0;
wire [`NUM_CORE-1:0] BVALID0;
wire [`NUM_CORE-1:0] ARREADY0;
wire [`NUM_CORE-1:0] RID0;
wire [`NUM_CORE-1:0] RLAST0;
wire [`NUM_CORE-1:0] RVALID0;
wire [`NUM_CORE-1:0] idle;
wire [`NUM_CORE-1:0] INT;
wire [`NUM_CORE-1:0] pslverr;
wire [`NUM_CORE-1:0] pready;
wire [`NUM_CORE-1:0] AWID0;
wire [`NUM_CORE-1:0] AWVALID0;
wire [`NUM_CORE-1:0] WID0;
wire [`NUM_CORE-1:0] WLAST0;
wire [`NUM_CORE-1:0] WVALID0;
wire [`NUM_CORE-1:0] BREADY0;
wire [`NUM_CORE-1:0] ARID0;
wire [`NUM_CORE-1:0] ARVALID0;
wire [`NUM_CORE-1:0] RREADY0;
wire [30:0] periph_tx_req [`NUM_CORE-1:0];
wire [30:0] periph_rx_req [`NUM_CORE-1:0];
wire [12:0] paddr [`NUM_CORE-1:0];
wire [31:0] pwdata [`NUM_CORE-1:0];
wire [1:0] BRESP0 [`NUM_CORE-1:0];
wire [63:0] RDATA0 [`NUM_CORE-1:0];
wire [1:0] RRESP0 [`NUM_CORE-1:0];
wire [30:0] periph_tx_clr [`NUM_CORE-1:0];
wire [30:0] periph_rx_clr [`NUM_CORE-1:0];
wire [31:0] prdata [`NUM_CORE-1:0];
wire [31:0] AWADDR0 [`NUM_CORE-1:0];
wire [3:0] AWLEN0 [`NUM_CORE-1:0];
wire [1:0] AWSIZE0 [`NUM_CORE-1:0];
wire [63:0] WDATA0 [`NUM_CORE-1:0];
wire [7:0] WSTRB0 [`NUM_CORE-1:0];
wire [31:0] ARADDR0 [`NUM_CORE-1:0];
wire [3:0] ARLEN0 [`NUM_CORE-1:0];
wire [1:0] ARSIZE0 [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({scan_en[i],psel[i],penable[i],pwrite[i],AWREADY0[i],WREADY0[i],BID0[i],BVALID0[i],ARREADY0[i],RID0[i],RLAST0[i],RVALID0[i],periph_tx_req[i],periph_rx_req[i],paddr[i],pwdata[i],BRESP0[i],RDATA0[i],RRESP0[i]}));

		oc_dma_axi64 oc_stamp(
			.clk(clk),
			.pclken(pclken),
			.reset(reset),
			.scan_en(scan_en[i]),
			.psel(psel[i]),
			.penable(penable[i]),
			.pwrite(pwrite[i]),
			.AWREADY0(AWREADY0[i]),
			.WREADY0(WREADY0[i]),
			.BID0(BID0[i]),
			.BVALID0(BVALID0[i]),
			.ARREADY0(ARREADY0[i]),
			.RID0(RID0[i]),
			.RLAST0(RLAST0[i]),
			.RVALID0(RVALID0[i]),
			.periph_tx_req(periph_tx_req[i]),
			.periph_rx_req(periph_rx_req[i]),
			.paddr(paddr[i]),
			.pwdata(pwdata[i]),
			.BRESP0(BRESP0[i]),
			.RDATA0(RDATA0[i]),
			.RRESP0(RRESP0[i]),
			.idle(idle[i]),
			.INT(INT[i]),
			.pslverr(pslverr[i]),
			.pready(pready[i]),
			.AWID0(AWID0[i]),
			.AWVALID0(AWVALID0[i]),
			.WID0(WID0[i]),
			.WLAST0(WLAST0[i]),
			.WVALID0(WVALID0[i]),
			.BREADY0(BREADY0[i]),
			.ARID0(ARID0[i]),
			.ARVALID0(ARVALID0[i]),
			.RREADY0(RREADY0[i]),
			.periph_tx_clr(periph_tx_clr[i]),
			.periph_rx_clr(periph_rx_clr[i]),
			.prdata(prdata[i]),
			.AWADDR0(AWADDR0[i]),
			.AWLEN0(AWLEN0[i]),
			.AWSIZE0(AWSIZE0[i]),
			.WDATA0(WDATA0[i]),
			.WSTRB0(WSTRB0[i]),
			.ARADDR0(ARADDR0[i]),
			.ARLEN0(ARLEN0[i]),
			.ARSIZE0(ARSIZE0[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({idle[i],INT[i],pslverr[i],pready[i],AWID0[i],AWVALID0[i],WID0[i],WLAST0[i],WVALID0[i],BREADY0[i],ARID0[i],ARVALID0[i],RREADY0[i],periph_tx_clr[i],periph_rx_clr[i],prdata[i],AWADDR0[i],AWLEN0[i],AWSIZE0[i],WDATA0[i],WSTRB0[i],ARADDR0[i],ARLEN0[i],ARSIZE0[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
